// EventFileViewerCallDllCpp.cpp : This file contains the 'main' function. Program execution begins and ends there.
//

#include <iostream>
#include <windows.h>
#include <string>
#include <iomanip>
#include "Event_File_Viewer_7_2_Example.h"
#include <thread>
#include <chrono>
#include <future>


int main()
{
	// N.B: Event_File_Reader_7_0.dll must be in the same folder as this executable
	//User settings  
	// Load the DLL
	LPCWSTR wrapperDll = L"Event_File_Reader_Dll_UM.dll";
	HINSTANCE hGetProcIDDLL = LoadLibrary(wrapperDll);

	//***************************************************Flat file format***************************************************//
	//Use this example if you are using flat file format (event files with extensions *.NLE). You can only look at a single file at a time for this file format
	
	/*	const int numberOfFiles = 1; //number of files you wish to load
		const int maxPathLength = 260;//do not modify and do not enter file paths longer than this number of characters
		char filesToLoad[numberOfFiles][maxPathLength] = { "C:\\EventLog.NLE" }; //event log file to load
		*/

	



	//*************************************************** Block file format*************************************************//
	// Use this example if you are using block file format (*.DF1) and would like to enter multiple files containing event data


	

		const int numberOfFiles = 10; //number of files you wish to load
		const int firstFile = 0;
		const int maxPathLength = 260;//do not modify and do not enter file paths longer than this number of characters
		char filesToLoad[numberOfFiles][maxPathLength];
		int padSize = 4;

		for (int i = firstFile; i < firstFile + numberOfFiles; i++)
		{
			std::string fileIdx = std::to_string(i);
			padTo(fileIdx, padSize, '0');
			std::string fileName = "G:\\NEUR" + fileIdx +".DF1";
			const char* fileNameChars = fileName.c_str();
			strcpy_s(filesToLoad[i], fileNameChars);
		}
	
			

	//**********************************************************************************************************************//

	int recordToRetrieve = 1; //the index of the record you would like to retrieve
	int maximumTimeToRunS = 10; //maximum time in seconds to allow function to run before cancelling. If want it to complete regardless of how long it takes, set this to zero.



	//initialize pointer array sizes - Do not modify
	const int maxCharactersInString = 512; //maximum characters in a field in a record. 
	const int numberOfFields = 6; //number of fields in each record. 
	char errorMessage[maxCharactersInString]; //the error message will be a char array of maximum length maxCharactersInString
	 
	// load in functions from dll
	typedef int(__cdecl *f_LoadFiles)(const char(*fileNames)[maxPathLength], int numberOfFiles, int cancelAfter);
	typedef void(__cdecl *f_GetErrorAsString)(int error, char* errorStrBuffer);
	typedef int(__cdecl *f_GetHeader)(char* header);
	typedef int(__cdecl *f_GetIndexedRecord)(int recordIndex, char(*ptrEventRecord)[maxCharactersInString]);
	typedef int(__cdecl *f_GetNumberOfRecords)(int* ptrNumberOfRecords);

	//check the library was loaded in successfully
	if (!hGetProcIDDLL) 
	{
		std::cout << "Could not load the dynamic library" << std::endl;
		FreeLibrary(hGetProcIDDLL);
		return EXIT_FAILURE;
	}  
		
	//load file
	f_LoadFiles LoadFiles = (f_LoadFiles)GetProcAddress(hGetProcIDDLL, "LoadFiles");

	//check each function was loaded in successfully
	if (!LoadFiles) 
	{
		std::cout << "Could not locate the function LoadFile" << std::endl;
		FreeLibrary(hGetProcIDDLL);
		return EXIT_FAILURE;
	}


	f_GetErrorAsString GetErrorAsString = (f_GetErrorAsString)GetProcAddress(hGetProcIDDLL, "GetErrorAsString");
	if (!GetErrorAsString) 
	{
		std::cout << "Could not locate the function GetErrorAsString" << std::endl;
		FreeLibrary(hGetProcIDDLL);
		return EXIT_FAILURE;
	}

	f_GetHeader GetHeader = (f_GetHeader)GetProcAddress(hGetProcIDDLL, "GetHeader");
	if (!GetHeader) 
	{
		std::cout << "Could not locate the function GetHeader" << std::endl;
		FreeLibrary(hGetProcIDDLL);
		return EXIT_FAILURE;
	}

	f_GetIndexedRecord GetIndexedRecord = (f_GetIndexedRecord)GetProcAddress(hGetProcIDDLL, "GetIndexedRecord");
	if (!GetIndexedRecord)
	{
		std::cout << "Could not locate function GetIndexedRecord" << std::endl;
		FreeLibrary(hGetProcIDDLL);
		return EXIT_FAILURE;
	}

	f_GetNumberOfRecords GetNumberOfRecords = (f_GetNumberOfRecords)GetProcAddress(hGetProcIDDLL, "GetNumberOfRecords");
	{
		if (!GetNumberOfRecords)
		{
			std::cout << "Could not locate function GetNumberOfRecords" << std::endl;
			FreeLibrary(hGetProcIDDLL);
			return EXIT_FAILURE;
		}
	}

	//load and retrieve data from file

	int error = LoadFiles(filesToLoad, numberOfFiles, maximumTimeToRunS);
	if (!error)
	{
		std::cout << "File loaded successfully.\n";
	}
	else
	{
		GetErrorAsString(error, errorMessage);
		std::cout << errorMessage << std::endl;
		FreeLibrary(hGetProcIDDLL);
		return EXIT_FAILURE;
	}

	//how many records in the file
	int numberOfRecords = 0;
	error = GetNumberOfRecords(&numberOfRecords);
	if (!error)
	{
		std::cout << "Number of records in this file: " << numberOfRecords << std::endl;
	}
	else
	{
		std::cout << "Unable to retrieve the number of records.\n";
		GetErrorAsString(error, errorMessage);
		std::cout << errorMessage << std::endl;
		FreeLibrary(hGetProcIDDLL);
		return EXIT_FAILURE;
	}

	//Get header from file - only for flat file format
	/*
	char header[maxCharactersInString];
	error = GetHeader(header);
	if (!error)
	{
		std::cout << "The header is: " << header << std::endl;
	}
	else
	{
		std::cout << "Unable to read the header\n";
		GetErrorAsString(error, errorMessage);
		std::cout << errorMessage << std::endl;
		FreeLibrary(hGetProcIDDLL);
		return EXIT_FAILURE;
	}
	*/

	
	//Get a specific record from the file
	char Record[numberOfFields][maxCharactersInString];
	error = GetIndexedRecord(recordToRetrieve, Record);
	if (!error)
	{
		std::cout << "Record " << recordToRetrieve << std::endl;
		std::cout << "Event number: " << Record[0] << std::endl;
		std::cout << "Event type: " << Record[1] << std::endl;
		std::cout << "Timestamp: " << Record[2] << std::endl;
		std::cout << "Timestamp (ms from midnight): " << Record[3] << std::endl;
		std::cout << "Time source: " << Record[4] << std::endl;
		std::cout << "Details: " << Record[5] << std::endl;

	}
	else
	{
		std::cout << "Unable to retrieve record\n";
		GetErrorAsString(error, errorMessage);
		std::cout << errorMessage << std::endl;
		FreeLibrary(hGetProcIDDLL);
		return EXIT_FAILURE;
	}
	FreeLibrary(hGetProcIDDLL);

	return EXIT_SUCCESS;
	


	
	
}

void padTo(std::string &str, const size_t num, const char paddingChar)
{
	if (num > str.size())
		str.insert(0, num - str.size(), paddingChar);
}



